from unittest.mock import patch

import pytest

from src.license_scanner.cli import Mode, main


@pytest.mark.parametrize(
    "allowed_licenses, allowed_packages, all_licenses",
    [
        (["BSD license"], [], {"MIT license": ["package-1", "package-2"]}),
        ([], ["package-1"], {"BSD license": ["package-1", "package-2"]}),
        (
            ["MIT license"],
            ["package-4"],
            {"MIT license": ["package-1", "package-2"], "BSD license": ["package-3"]},
        ),
    ],
    ids=[
        "not allowed licenses",
        "not allowed packages",
        "not allowed licenses and packages",
    ],
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.get_all_licenses")
@patch("src.license_scanner.cli.tomllib.load")
def test_white_listed_licenses_fail(
    toml_load_mock,
    get_all_licenses_mock,
    argparse_mock,
    allowed_licenses,
    allowed_packages,
    all_licenses,
):
    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    get_all_licenses_mock.return_value = all_licenses
    argparse_mock.return_value = Mode.whitelist

    with pytest.raises(ValueError):
        main()


@pytest.mark.parametrize(
    "allowed_licenses, allowed_packages, all_licenses",
    [
        (["BSD license"], [], {"BSD license": ["package_1", "package-2"]}),
        ([], ["package-1", "package_2"], {"BSD license": ["package_1", "package-2"]}),
        (
            ["MIT license"],
            ["package-3"],
            {"MIT license": ["package-1", "package-2"], "BSD license": ["package-3"]},
        ),
        (
            ["MIT", "BSD", "apache 2.0"],
            [],
            {
                "MIT license": ["package-1"],
                "BSD license": ["package-2"],
                "Apache license 2.0": ["package-3"],
            },
        ),
        (
            [
                'BSD 4-Clause "Original" or "Old" License',
                'BSD 3-Clause "New" or "Revised" License',
                "GNU Affero General Public License v1.0 or later",
                "Historical Permission Notice and Disclaimer (HPND)",
                "FSF Unlimited License (With License Retention and Warranty Disclaimer)",
            ],
            [],
            {
                'BSD 4-Clause "Original" or "Old" License': ["package-1"],
                'BSD 3-Clause "New" or "Revised" License': ["package-2"],
                "GNU Affero General Public License v1.0 or later": ["package-3"],
                "Historical Permission Notice and Disclaimer (HPND)": ["package-4"],
                "FSF Unlimited License (With License Retention and Warranty Disclaimer)": [
                    "package-5"
                ],
            },
        ),
    ],
    ids=[
        "allowed licenses",
        "allowed packages",
        "allowed licenses and packages",
        "synonyms",
        "licenses with OR/AND it in the name",
    ],
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.get_all_licenses")
@patch("src.license_scanner.cli.tomllib.load")
def test_white_listed_licenses(
    toml_load_mock,
    get_all_licenses_mock,
    argparse_mock,
    allowed_licenses,
    allowed_packages,
    all_licenses,
):
    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    get_all_licenses_mock.return_value = all_licenses
    argparse_mock.return_value = Mode.whitelist

    main()



import os
import warnings
from typing import Union
from .licenses_synonyms import LICENSES_SYNONYMS, unknown_license

current_loc = os.path.dirname(os.path.realpath(__file__))


def parse_license(license_str: str) -> Union[str, None]:
    """Turn a general license name into one of that is in the list
    This makes it easier to compare license that might be differently written.

    If the license string seems to be a full license, None is returned.
    If the license is not found in the list of known licenses, a warning is raised
    and the original string is returned.

    :param license_str: license string found in package metadata
    :type license_str: str
    :return: normalized license string
    :rtype: str | None
    """
    # Get license
    if not license_str:
        return None

    # If a break line is in there, it is a whole license.
    # Don't bother
    if "\n" in license_str:
        # Only print the first 5 lines
        part_of_license = repr("\n".join(license_str.split("\n")[:5]))
        warnings.warn(
            f"It seems like a full license (not just a name) was paste for license: {part_of_license}"
        )
        return None

    # Make case insensitive
    license_str = license_str.lower()

    if "copyright (c) " in license_str:
        license_str = license_str.split("copyright (c) ")[0]

    if len(license_str) > 300:
        license_str = license_str[:300]

    # Strip whitespace, this is done last to not interfere with full license check
    license_str = license_str.strip()

    license_normalized = LICENSES_SYNONYMS.get(license_str)
    if license_normalized is None:
        warnings.warn(
            f'The license "{license_str}" was not found in list of known licenses'
        )
        license_normalized = license_str

    # Replace empty with unknown
    license_normalized = license_normalized if license_normalized else unknown_license
    return license_normalized



from __future__ import annotations

import sys

import os
import re
from typing import List


def safe_listdir(path):
    """
    Attempt to list contents of path, but suppress some exceptions.
    """
    if not os.path.isdir(path):
        return ()
    return os.listdir(path)


EGG_NAME = re.compile(
    r"""
    (?P<name>[^-]+) (
        -(?P<ver>[^-]+) (
            -py(?P<pyver>[^-]+) (
                -(?P<plat>.+)
            )?
        )?
    )?
    """,
    re.VERBOSE | re.IGNORECASE,
).match


def get_package_name(basename) -> str:
    match = EGG_NAME(basename)
    if match:
        project_name = match.group("name")
        return project_name
    return None


def _cygwin_patch(filename):  # pragma: nocover
    """
    Contrary to POSIX 2008, on Cygwin, getcwd (3) contains
    symlink components. Using
    os.path.abspath() works around this limitation. A fix in os.getcwd()
    would probably better, in Cygwin even more so, except
    that this seems to be by design...
    """
    return os.path.abspath(filename) if sys.platform == "cygwin" else filename


def normalize_path(filename) -> str | bytes:
    """Normalize a file/dir name for comparison purposes"""
    return os.path.normcase(os.path.realpath(os.path.normpath(_cygwin_patch(filename))))


def get_all_package_names() -> List[str]:
    """Get all package names installed into the environment

    :return: list of package names
    :rtype: List[str]
    """
    all_package_names = []

    for sys_path in sys.path:
        sys_path = normalize_path(sys_path)
        all_modules = safe_listdir(sys_path)
        for module_folder_name in all_modules:
            lower = str(module_folder_name).lower()
            is_egg_info = lower.endswith(".egg-info")
            if is_egg_info:
                package_name = module_folder_name[: -len(".egg-info")]
            is_dist_info = lower.endswith(".dist-info") and os.path.isdir(
                os.path.join(sys_path, module_folder_name)
            )
            if is_dist_info:
                package_name = module_folder_name[: -len(".dist-info")]

            if is_egg_info or is_dist_info:
                package_name: str = get_package_name(package_name)
                if package_name:
                    all_package_names.append(package_name)
                else:
                    print(f"Could not parse package name from {module_folder_name}")
                    all_package_names.append(module_folder_name)

    return all_package_names


if __name__ == "__main__":
    all_package_names = get_all_package_names()

    print(f"All package names {len(all_package_names)}:", sorted(all_package_names))
    print(f"Are there unknowns: {'Unknown' in all_package_names}")



from unittest.mock import patch

import pytest

from src.license_scanner.parse_license import parse_license
from src.license_scanner.cli import Mode, main
from src.license_scanner.parse_license.licenses_synonyms import LICENSES_SYNONYMS

ALL_KNOWN_LICENSES = list(LICENSES_SYNONYMS.keys())
ALL_ALLOWED_LICENSES = list(set(LICENSES_SYNONYMS.values()))


@pytest.mark.parametrize(
    "license_name",
    ALL_KNOWN_LICENSES,
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.get_all_licenses")
@patch("src.license_scanner.cli.tomllib.load")
def test_all_licenses_positive(
    toml_load_mock,
    get_all_licenses_mock,
    argparse_mock,
    license_name,
):
    """
    Test for EVERY license synonym known if it would create a positive hit
    """
    allowed_licenses = [license_name]
    allowed_packages = []
    all_licenses = {license_name: ["package 1", "package 2"]}

    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    get_all_licenses_mock.return_value = all_licenses
    argparse_mock.return_value = Mode.whitelist

    main()


@pytest.mark.parametrize(
    "license_name",
    ALL_KNOWN_LICENSES,
)
@patch("src.license_scanner.cli.__get_arguments")
@patch("src.license_scanner.cli.get_all_licenses")
@patch("src.license_scanner.cli.tomllib.load")
def test_all_licenses_negative(
    toml_load_mock,
    get_all_licenses_mock,
    argparse_mock,
    license_name,
):
    """
    Test for EVERY license synonym known if it would create a negative hit
    """
    allowed_licenses = ALL_ALLOWED_LICENSES.copy()
    allowed_licenses.remove(parse_license(license_name))
    allowed_packages = []
    all_licenses = {license_name: ["package 1", "package 2"]}

    toml_load_mock.return_value = {
        "tool": {
            "license_scanner": {
                "allowed_licenses": allowed_licenses,
                "allowed_packages": allowed_packages,
            }
        }
    }
    get_all_licenses_mock.return_value = all_licenses
    argparse_mock.return_value = Mode.whitelist

    with pytest.raises(ValueError):
        main()



# -*- coding: utf-8 -*-
"""Scans your environment for all needed licenses"""

__author__ = """Tom Nijhof-Verhees"""
__version__ = "0.4.3"

# Add here import to all the functions you need

from .get_all_licenses import get_all_licenses
